//
//		(c) 2008, .. .    .  .
//
//
//			    :
//
//			- PB[0] -  SDA (/)
//			- PB[1] -    ()
//			- PB[2] -  SCL ()
//			- PB[3] -  " " ()
//			- PB[4] -   "" ()
//

 
#include "avr/io.h"
#include "avr/interrupt.h"
#include <avr/pgmspace.h>
#include "avr/eeprom.h"
#include "avr/sleep.h"
#include "util/delay.h"
#include "util/twi.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdarg.h>


#define SONY	1					//   ""  Sony.

#define	SDA		PB0					//   .
#define	SCL		PB1
#define	LIGHT	PB2
#define	SETUP	PB3
#define	INFRA	PB4

//
//			   ,   .
//

                //  .
#define T_ONOFF_ON      0x00   		// ON/OFF
#define T_MUTE_ON       0x01        // MUTE
#define T_VOL_P_ON      0x02        // VOL +
#define T_VOL_M_ON      0x03        // VOL -
#define T_CH_P_ON       0x04        // CH +
#define T_CH_M_ON       0x05        // CH -
#define T_MENU_ON       0x06        // MENU
#define T_DELAY         0x0A        //   .
#define T_TIMEOUT       0x0B        //  .
#define T_DOWN          0x00        //  "".
#define T_UP            0x10        //  "".
#define T_REPEAT        0x20        //  "".


                //   .
#define X_NOTHING       0x00		// .
#define X_VOL_P         0x01        //   1.
#define X_VOL_M         0x02        //   1.
#define X_VOL_OFF       0x03        //   1.
#define X_BASS_P        0x04        //   2.
#define X_BASS_M        0x05        //   2.
#define X_BASS_OFF      0x06        //   2.
#define X_TREBLE_P      0x07        //   3.
#define X_TREBLE_M      0x08        //   3.
#define X_TREBLE_OFF    0x09        //   3.
#define X_CH_P          0x0A        //  .
#define X_CH_M          0x0B        //  .
#define X_TO_VOL        0x0C        //   1.
#define X_TO_BASS       0x0D        //   2.
#define X_TO_TREBLE     0x0E        //   3.
#define X_TURNON        0x0F        //  .
#define X_TURNOFF       0x10        //  .
#define X_MUTE_ON       0x11        //  .
#define X_MUTE_OFF      0x12        //  .
#define X_CLEAR_LED     0x13        //  LED-
#define X_SET_LED       0x14        //  LED-
#define X_BLINK_ON      0x15        //    .
#define X_BLINK_OFF     0x16        //    .

#define TO_SECOND		30			//        0.

typedef struct			//  .
	{unsigned char		actualState;	//   .
	 unsigned char		terminal;		//  .
	 unsigned char		newState;		//   .
	 unsigned char		action;			//  .
	} AutomatLine;

PROGMEM AutomatLine		table[] = 		//   -       .
	{{0,      T_TIMEOUT,              16,     X_BLINK_ON},
	 {16,     T_TIMEOUT,              0,      X_BLINK_OFF},
	 {0,      T_ONOFF_ON,             1,      X_TURNON},
     {1,      T_ONOFF_ON,             0,      X_TURNOFF},
     {2,      T_ONOFF_ON,             0,      X_TURNOFF},
     {3,      T_ONOFF_ON,             0,      X_TURNOFF},
     {10,     T_ONOFF_ON,             0,      X_TURNOFF},
     {11,     T_ONOFF_ON,             0,      X_TURNOFF},
     {12,     T_ONOFF_ON,             0,      X_TURNOFF},
     {1,      T_CH_P_ON,              2,      X_TO_BASS},
     {2,      T_CH_P_ON,              3,      X_TO_TREBLE},
     {3,      T_CH_P_ON,              1,      X_TO_VOL},
     {1,      T_CH_M_ON,              3,      X_TO_TREBLE},
     {3,      T_CH_M_ON,              2,      X_TO_BASS},
     {2,      T_CH_M_ON,              1,      X_TO_VOL},
     {1,      T_VOL_P_ON,             4,      X_VOL_P},
     {4,      T_VOL_P_ON|T_UP,        1,      X_VOL_OFF},
     {1,      T_VOL_M_ON,             5,      X_VOL_M},
     {5,      T_VOL_M_ON|T_UP,        1,      X_VOL_OFF},
     {2,      T_VOL_P_ON,             6,      X_BASS_P},
     {6,      T_VOL_P_ON|T_UP,        2,      X_BASS_OFF},
     {2,      T_VOL_M_ON,             7,      X_BASS_M},
     {7,      T_VOL_M_ON|T_UP,        2,      X_BASS_OFF},
     {3,      T_VOL_P_ON,             8,      X_TREBLE_P},
     {8,      T_VOL_P_ON|T_UP,        3,      X_TREBLE_OFF},
     {3,      T_VOL_M_ON,             9,      X_TREBLE_M},
     {9,      T_VOL_M_ON|T_UP,        3,      X_TREBLE_OFF},
     {1,      T_MENU_ON,              10,     X_CLEAR_LED},
     {10,     T_MENU_ON,              1,      X_SET_LED},
     {10,     T_CH_P_ON,              10,     X_CH_P},
     {10,     T_CH_M_ON,              10,     X_CH_M},
     {2,      T_MENU_ON,              11,     X_CLEAR_LED},
     {11,     T_MENU_ON,              2,      X_SET_LED},
     {11,     T_CH_P_ON,              11,     X_CH_P},
     {11,     T_CH_M_ON,              11,     X_CH_M},
     {3,      T_MENU_ON,              12,     X_CLEAR_LED},
     {12,     T_MENU_ON,              3,      X_SET_LED},
     {12,     T_CH_P_ON,              12,     X_CH_P},
     {12,     T_CH_M_ON,              12,     X_CH_M},
     {1,      T_MUTE_ON,              13,     X_MUTE_ON},
     {13,     T_MUTE_ON,              1,      X_MUTE_OFF},
     {13,     T_ONOFF_ON,             0,      X_TURNOFF},
     {2,      T_MUTE_ON,              14,     X_MUTE_ON},
     {14,     T_MUTE_ON,              2,      X_MUTE_OFF},
     {14,     T_ONOFF_ON,             0,      X_TURNOFF},
     {3,      T_MUTE_ON,              15,     X_MUTE_ON},
     {15,     T_MUTE_ON,              3,      X_MUTE_OFF},
     {15,     T_ONOFF_ON,             0,      X_TURNOFF}
	};


//
//			 
//

				//   .
#define	LED3_ADDRESS		0x60	//       .
#define	RELE3_ADDRESS		0x64	//   .
#define	LG_VOL_ADDRESS		0x68	//   .
#define	LG_BASS_ADDRESS		0x69	//   .
#define	LG_TREBLE_ADDRESS	0x6A	//   .
#define	ONOFF_ADDRESS		0x70	//    .
#define	MOTOR_ADDRESS		0x74	//    .
#define DELAY_NOW			0xFF	//   ,    !

#define _RESET				0x00	//  .
#define _PHONE_ON			0x11	//   .
#define _PHONE_OFF			0x10	//   .
#define _ON_BYMODE			0x12	//   ()  .
#define _OFF_BYMODE			0x13	//   ()  .
#define _STOP				0x11	//  .
#define _LESS				0x12	//     .
#define _GREATER			0x13	//     .
#define _SELECT				0x20	//  ,   .

#define _1					0x01	//  
#define _2					0x02
#define _3					0x04
#define _LEFT				0x01	//   .
#define _POINT				0x02
#define _RIGHT				0x04

typedef struct			//    .
	{unsigned char			address;	//  .
	 unsigned char			command;	//  .
	} CommandSet;

PROGMEM CommandSet			volUpSeq[] = //   .
	{{LG_VOL_ADDRESS,		_SELECT|_LEFT|_POINT},	//    .
	 {LG_VOL_ADDRESS,		_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_GREATER} 				//     .
	};

PROGMEM CommandSet			volDownSeq[] = //   .
	{{LG_VOL_ADDRESS,		_SELECT|_RIGHT|_POINT},	//    .
	 {LG_VOL_ADDRESS,		_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_LESS} 					//     .
	};

PROGMEM CommandSet			volStopSeq[] = //   .
	{{LG_VOL_ADDRESS,		_SELECT|_POINT},		//    .
	 {LG_VOL_ADDRESS,		_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_STOP} 					//  .
	};

PROGMEM CommandSet			bassUpSeq[] = //   .
	{{LG_BASS_ADDRESS,		_SELECT|_LEFT|_POINT},	//    .
	 {LG_BASS_ADDRESS,		_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_GREATER} 				//     .
	};

PROGMEM CommandSet			bassDownSeq[] = //   .
	{{LG_BASS_ADDRESS,		_SELECT|_RIGHT|_POINT},	//    .
	 {LG_BASS_ADDRESS,		_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_LESS} 					//     .
	};

PROGMEM CommandSet			bassStopSeq[] = //   .
	{{LG_BASS_ADDRESS,		_SELECT|_POINT},		//    .
	 {LG_BASS_ADDRESS,		_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_STOP} 					//  .
	};

PROGMEM CommandSet			trebleUpSeq[] = //   .
	{{LG_TREBLE_ADDRESS,	_SELECT|_LEFT|_POINT},	//    .
	 {LG_TREBLE_ADDRESS,	_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_GREATER} 				//     .
	};

PROGMEM CommandSet			trebleDownSeq[] = //   .
	{{LG_TREBLE_ADDRESS,	_SELECT|_RIGHT|_POINT},	//    .
	 {LG_TREBLE_ADDRESS,	_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_LESS} 					//     .
	};

PROGMEM CommandSet			trebleStopSeq[] = //   .
	{{LG_TREBLE_ADDRESS,	_SELECT|_POINT},		//    .
	 {LG_TREBLE_ADDRESS,	_ON_BYMODE},			//    .
	 {MOTOR_ADDRESS,		_STOP} 					//  .
	};

PROGMEM CommandSet			channelSeq[][4] = //   .
	{{
	 {LED3_ADDRESS,			_SELECT|_1},	//    .
	 {RELE3_ADDRESS,		_SELECT|_1},	//    .
	 {LED3_ADDRESS,			_ON_BYMODE},	//   .
	 {RELE3_ADDRESS,		_ON_BYMODE}		//   .
	 },
	 {
	 {LED3_ADDRESS,			_SELECT|_2},	//    .
	 {RELE3_ADDRESS,		_SELECT|_2},	//    .
	 {LED3_ADDRESS,			_ON_BYMODE},	//   .
	 {RELE3_ADDRESS,		_ON_BYMODE}		//   .
	 },
	 {
	 {LED3_ADDRESS,			_SELECT|_3},	//    .
	 {RELE3_ADDRESS,		_SELECT|_3},	//    .
	 {LED3_ADDRESS,			_ON_BYMODE},	//   .
	 {RELE3_ADDRESS,		_ON_BYMODE}		//   .
	 },
	};

PROGMEM CommandSet			selectVolSeq[] = //   .
	{{LG_VOL_ADDRESS,		_SELECT|_POINT},//   .
	 {LG_VOL_ADDRESS,		_ON_BYMODE},	//   .
	 {LG_BASS_ADDRESS,		_OFF_BYMODE},	//   .
	 {LG_TREBLE_ADDRESS,	_OFF_BYMODE},	//   .
	 {MOTOR_ADDRESS,		_SELECT|_1} 	//    .
	};

PROGMEM CommandSet			selectBassSeq[] = //   .
	{{LG_VOL_ADDRESS,		_OFF_BYMODE},	//   .
	 {LG_BASS_ADDRESS,		_SELECT|_POINT},//   .
	 {LG_BASS_ADDRESS,		_ON_BYMODE},	//   .
	 {LG_TREBLE_ADDRESS,	_OFF_BYMODE},	//   .
	 {MOTOR_ADDRESS,		_SELECT|_2} 	//    .
	};

PROGMEM CommandSet			selectTrebleSeq[] = //   .
	{{LG_VOL_ADDRESS,		_OFF_BYMODE},	//   .
	 {LG_BASS_ADDRESS,		_OFF_BYMODE},	//   .
	 {LG_TREBLE_ADDRESS,	_SELECT|_POINT},//   .
	 {LG_TREBLE_ADDRESS,	_ON_BYMODE},	//   .
	 {MOTOR_ADDRESS,		_SELECT|_3} 	//    .
	};

PROGMEM CommandSet			turnOnSeq[] = //   .
	{{ONOFF_ADDRESS,		_PHONE_ON},			//   .
	 {LED3_ADDRESS,			_SELECT|_1},		//    .
	 {RELE3_ADDRESS,		_SELECT|_1},		//    .
	 {LG_VOL_ADDRESS,		_SELECT|_POINT},	//    .
	 {LG_BASS_ADDRESS,		_SELECT|0},			//    .
	 {LG_TREBLE_ADDRESS,	_SELECT|0},			//    .
	 {MOTOR_ADDRESS,		_SELECT|_1},		//    .
	 {DELAY_NOW,			6},					//   2 .
	 {LED3_ADDRESS,			_PHONE_ON},			//   3- .
	 {LG_VOL_ADDRESS,		_PHONE_ON},			//    .
	 {LG_BASS_ADDRESS,		_PHONE_ON},			//    .
	 {LG_TREBLE_ADDRESS,	_PHONE_ON},			//    .
	 {LG_VOL_ADDRESS,		_ON_BYMODE},		//   .
	 {LED3_ADDRESS,			_ON_BYMODE},		//   .
	 {RELE3_ADDRESS,		_ON_BYMODE}			//   .
	};

PROGMEM CommandSet			turnOffSeq[] = //   .
	{{LED3_ADDRESS,			_OFF_BYMODE},		//     .
	 {LG_VOL_ADDRESS,		_OFF_BYMODE},		//     .
	 {LG_BASS_ADDRESS,		_OFF_BYMODE},		//     .
	 {LG_TREBLE_ADDRESS,	_OFF_BYMODE},		//     .
	 {RELE3_ADDRESS,		_OFF_BYMODE},		//   .
	 {LED3_ADDRESS,			_PHONE_OFF},		//  .
	 {LG_VOL_ADDRESS,		_PHONE_OFF},		//  .
	 {LG_BASS_ADDRESS,		_PHONE_OFF},		//  .
	 {LG_TREBLE_ADDRESS,	_PHONE_OFF},		//  .
	 {ONOFF_ADDRESS,		_PHONE_OFF}			//   .
	};

PROGMEM CommandSet			muteOnSeq[] = //    "MUTE".
	{{LED3_ADDRESS,			_OFF_BYMODE},		//     .
	 {RELE3_ADDRESS,		_OFF_BYMODE}		//   .
	};

PROGMEM CommandSet			muteOffSeq[] = //    "MUTE".
	{{RELE3_ADDRESS,		_ON_BYMODE},		//  .
	 {LED3_ADDRESS,			_ON_BYMODE}			//     .
	};

PROGMEM CommandSet			ledOffSeq[] = //     .
	{{LG_VOL_ADDRESS,		_OFF_BYMODE},		//    .
	 {LG_BASS_ADDRESS,		_OFF_BYMODE},		//    .
	 {LG_TREBLE_ADDRESS,	_OFF_BYMODE}		//    .
	};

PROGMEM CommandSet			ledOnSeq[] = //     .
	{{LG_VOL_ADDRESS,		_ON_BYMODE},		//    .
	 {LG_BASS_ADDRESS,		_ON_BYMODE},		//    .
	 {LG_TREBLE_ADDRESS,	_ON_BYMODE}			//    .
	};


//
//			 .
//


unsigned short 			templ[7];	//    .
unsigned char				actualState = 0, actualChannel = 0, onesCount = 0;
unsigned short 			tickCounter, delayTimer = 0, mainTimer = 0;
volatile unsigned short	package, tmppak;
volatile bool				received = false, delay = false, timeout = false;


//
//			 .
//

#ifdef SONY

ISR(PCINT0_vect)
	//    .
{	if (!(PINB & (1 << INFRA)))		//   .
		{if (TCCR1 & 0x0F) 			//     -    .
			TCCR1 = 0b00000000;			//  .
		 else 							//  -,     .
			{tickCounter = 13;			// 12   + 1    .
			 tmppak = 0;
			}
		 GTCCR |= 0b00000010;			//    1.
		 TCNT1 = 0;						//  .
		 TCCR1 = 0b00001101;			//   1    .  4096.
		}
	else	// c   -  .
		{if (!--tickCounter)			//  .
			{TCCR1 = 0b00000000;		//  .
			 package = tmppak;
			 received = true;
			}
		 else tmppak = (tmppak >> 1) | ((TCNT1 > 1) ? 0x8000 : 0x00);
		}	
}

ISR(TIM1_COMPA_vect){}
	//     .

#else

ISR(PCINT0_vect)
	//    .
{	if (!(PINB & (1 << INFRA)))		//  ,  ...
		{TCCR1 = 0b10000010;			//   1  .
		 GIMSK = 0b00000000;			// .       .
		 tickCounter = 14 * 32 + 1;		// 14   ( 32 ) + 1     .
		 tmppak = onesCount = 0;
		}
}


ISR(TIM1_COMPA_vect)
	//     .
{	if (!--tickCounter)			//  .
		{GIFR  = 0b00100000;		//    .
		 GIMSK = 0b00100000;		//     .
		 TCCR1 = 0b10000000;		//  .
		 package = tmppak & 0x07FF;	//       .
		 received = true;
		}
	else if ((tickCounter & 0x1F) > 0x18)		//       B (8 )!
		onesCount += ((PINB & (1 << INFRA)) ? 0x00 : 0x01);
	else if ((tickCounter & 0x1F) == 0x18)		//     ,   ,      .
		{tmppak = (tmppak << 1) | ((onesCount >= 4) ? 0x01 : 0x00);
		 onesCount = 0;
		}
}
#endif



ISR(TIM0_OVF_vect)
	//     .
{	//PORTB ^= (1 << LIGHT);
	if (delayTimer != 0) if (!--delayTimer) delay = true;
	if (mainTimer != 0) if (!--mainTimer) timeout = true;
}


//
//			    .
//

void d(void){int x; for (x = 0; x < 5; x++) _delay_ms(100);}
	//       ...

void d2(void){_delay_us(250);}
	//      ...

void sendCommand(unsigned char address,unsigned char command)
	//     i2c.
{unsigned char	index;

	PORTB &= ~(1 << SDA); d2();	//   .
	PORTB &= ~(1 << SCL); d2();
	
	for (index = 0; index < 8; index++, address <<= 1)	//     R/W
		{if (address & 0x80) PORTB |= 1 << SDA; else PORTB &= ~(1 << SDA);
		 d2();	PORTB |= 1 << SCL; d2();	PORTB &= ~(1 << SCL);
		}
		
	PORTB |= 1 << SDA;	DDRB &= ~(1 << SDA);	d2();	PORTB |= 1 << SCL; d2();	PORTB &= ~(1 << SCL);  DDRB |= 1 << SDA;	d2();	//   ACK.
	
	for (index = 0; index < 8; index++, command <<= 1)	//  
		{if (command & 0x80) PORTB |= 1 << SDA; else PORTB &= ~(1 << SDA);
		 d2();	PORTB |= 1 << SCL; d2();	PORTB &= ~(1 << SCL);
		}
		
	PORTB |= 1 << SDA;	DDRB &= ~(1 << SDA);	d2();	PORTB |= 1 << SCL; d2();	PORTB &= ~(1 << SCL);  DDRB |= 1 << SDA;	d2();	//   ACK.
	
	PORTB &= ~(1 << SDA);	d2();	PORTB |= 1 << SCL; d2();	PORTB |= 1 << SDA;	//   .
	
	d2();	d2(); 	//   .
}


void sendGroup(CommandSet *seq,unsigned char size)
	//    .
{unsigned char 	index, delay;
	
	for (index = 0; index < size; index++, seq++)		//    .
		if (pgm_read_byte_near(&seq->address) != DELAY_NOW)
			sendCommand(pgm_read_byte_near(&seq->address),pgm_read_byte_near(&seq->command));
		else for (delay = 0; delay < pgm_read_byte_near(&seq->command); delay++) d();
}	


void loadTemplates(void)
	//        templ.
{unsigned char index, lowByte;
 
	for (index = 0; index < sizeof(templ)/sizeof(templ[0]); index++)
		{while(EECR & (1<<EEPE));		EEAR = 2 * index;		EECR |= (1<<EERE);		lowByte = EEDR;
		 while(EECR & (1<<EEPE));		EEAR = 2 * index + 1;	EECR |= (1<<EERE);		templ[index] = (EEDR << 8) | lowByte;
		}
}	


void saveTemplates(void)
	//      templ  .
{unsigned char index;
 
	for (index = 0; index < sizeof(templ)/sizeof(templ[0]); index++)
		{while(EECR & (1<<EEPE));		EEAR = 2 * index;		EEDR = templ[index] & 0xFF;		EECR = (1<<EEMPE);	EECR |= (1<<EEPE);
		 while(EECR & (1<<EEPE));		EEAR = 2 * index + 1;	EEDR = templ[index] >> 8;		EECR = (1<<EEMPE);	EECR |= (1<<EEPE);
		}
}	


void indicateTemplNumber(unsigned char number)
	//       .
{	sendCommand(LED3_ADDRESS,0x20|(number & 0x07));
	sendCommand(LED3_ADDRESS,0x12);
}


void automat(unsigned char terminal)
	//    .
{unsigned char	index;

	for (index = 0; index < sizeof(table)/sizeof(table[0]); index++)
		if (actualState == pgm_read_byte_near(&table[index].actualState) && terminal == pgm_read_byte_near(&table[index].terminal))
			{actualState = pgm_read_byte_near(&table[index].newState);
			 switch (pgm_read_byte_near(&table[index].action))	//     .
				{case X_VOL_P://   1.
					sendGroup((CommandSet *)volUpSeq,sizeof(volUpSeq)/sizeof(volUpSeq[0]));						break;
				 case X_VOL_M://   1.
					sendGroup((CommandSet *)volDownSeq,sizeof(volDownSeq)/sizeof(volDownSeq[0]));				break;
				 case X_VOL_OFF://   1.
					sendGroup((CommandSet *)volStopSeq,sizeof(volStopSeq)/sizeof(volStopSeq[0]));				break;
				 case X_BASS_P://   2.
					sendGroup((CommandSet *)bassUpSeq,sizeof(bassUpSeq)/sizeof(bassUpSeq[0]));					break;
				 case X_BASS_M://   2.
					sendGroup((CommandSet *)bassDownSeq,sizeof(bassDownSeq)/sizeof(bassDownSeq[0]));				break;
				 case X_BASS_OFF://   2.
					sendGroup((CommandSet *)bassStopSeq,sizeof(bassStopSeq)/sizeof(bassStopSeq[0]));				break;
				 case X_TREBLE_P://   3.
					sendGroup((CommandSet *)trebleUpSeq,sizeof(trebleUpSeq)/sizeof(trebleUpSeq[0]));				break;
				 case X_TREBLE_M://   3.
					sendGroup((CommandSet *)trebleDownSeq,sizeof(trebleDownSeq)/sizeof(trebleDownSeq[0]));		break;
				 case X_TREBLE_OFF://   3.
					sendGroup((CommandSet *)trebleStopSeq,sizeof(trebleStopSeq)/sizeof(trebleStopSeq[0]));		break;
				 case X_CH_P://  .
					if (actualChannel == 2) actualChannel = 0; else actualChannel++;
					sendGroup((CommandSet *)channelSeq[actualChannel],sizeof(channelSeq[actualChannel])/sizeof(channelSeq[actualChannel][0]));
					break;
				 case X_CH_M://  .
					if (actualChannel == 0) actualChannel = 2; else actualChannel--;
					sendGroup((CommandSet *)channelSeq[actualChannel],sizeof(channelSeq[actualChannel])/sizeof(channelSeq[actualChannel][0]));
					break;
				 case X_TO_VOL://   1.
					sendGroup((CommandSet *)selectVolSeq,sizeof(selectVolSeq)/sizeof(selectVolSeq[0]));			break;
					break;
				 case X_TO_BASS://   2.
					sendGroup((CommandSet *)selectBassSeq,sizeof(selectBassSeq)/sizeof(selectBassSeq[0]));		break;
				 case X_TO_TREBLE://   3.
					sendGroup((CommandSet *)selectTrebleSeq,sizeof(selectTrebleSeq)/sizeof(selectTrebleSeq[0]));	break;
				 case X_TURNON://  .
					sendGroup((CommandSet *)turnOnSeq,sizeof(turnOnSeq)/sizeof(turnOnSeq[0]));			
					actualChannel = 1;					PORTB |= 1 << LIGHT;					
					break;
				 case X_TURNOFF://  .
					sendGroup((CommandSet *)turnOffSeq,sizeof(turnOffSeq)/sizeof(turnOffSeq[0]));
					PORTB &= ~(1 << LIGHT);	cli();	mainTimer = 30 * TO_SECOND;	sei();
					break;
				 case X_MUTE_ON://  .
					sendGroup((CommandSet *)muteOnSeq,sizeof(muteOnSeq)/sizeof(muteOnSeq)[0]);					break;
				 case X_MUTE_OFF://  .
					sendGroup((CommandSet *)muteOffSeq,sizeof(muteOffSeq)/sizeof(muteOffSeq[0]));				break;
				 case X_CLEAR_LED://  LED-
					sendGroup((CommandSet *)ledOffSeq,sizeof(ledOffSeq)/sizeof(ledOffSeq)[0]);					break;
				 case X_SET_LED://  LED-
					sendGroup((CommandSet *)ledOnSeq,sizeof(ledOnSeq)/sizeof(ledOnSeq[0]));						break;
				 case X_BLINK_ON://    .
					PORTB |= 1 << LIGHT;		cli();	mainTimer = TO_SECOND / 4;	sei();							break;
				 case X_BLINK_OFF://    .				
					PORTB &= ~(1 << LIGHT);	cli();	mainTimer = 30 * TO_SECOND;	sei();							break;
				}
			 return;
			}
}	


int main(int argc, char *argv[])
	//   .
{//signed int	value;
 unsigned char		index, oldTerminal = 0xFF;
 unsigned short	oldPackage = 0, newPackage;

	d();						// ,     .
	CLKPR  = 0b10000000;		// !
	CLKPR  = 0b00000000;		//     8!

	PORTB = (1 << SDA) | (1 << SCL) | (1 << SETUP);	//    B.
	DDRB  = (1 << SDA) | (1 << SCL) | (1 << LIGHT);
	
/*xxx:		//  -       ...

	sendCommand(LED3_ADDRESS,0x11);			d();	//  3-    .
	sendCommand(LED3_ADDRESS,0x10);			d();
	sendCommand(LED3_ADDRESS,0x11);			d();
	sendCommand(LED3_ADDRESS,0x21);			d();
	sendCommand(LED3_ADDRESS,0x12);			d();
	sendCommand(LED3_ADDRESS,0x13);			d();
	sendCommand(LED3_ADDRESS,0x10);			d();
	
	sendCommand(LG_VOL_ADDRESS,0x11);		d();	//  3-    .
	sendCommand(LG_VOL_ADDRESS,0x10);		d();
	sendCommand(LG_VOL_ADDRESS,0x11);		d();
	sendCommand(LG_VOL_ADDRESS,0x27);		d();
	sendCommand(LG_VOL_ADDRESS,0x12);		d();
	sendCommand(LG_VOL_ADDRESS,0x13);		d();
	sendCommand(LG_VOL_ADDRESS,0x10);		d();
	
	sendCommand(LG_BASS_ADDRESS,0x11);		d();	//  3-    .
	sendCommand(LG_BASS_ADDRESS,0x10);		d();
	sendCommand(LG_BASS_ADDRESS,0x11);		d();
	sendCommand(LG_BASS_ADDRESS,0x27);		d();
	sendCommand(LG_BASS_ADDRESS,0x12);		d();
	sendCommand(LG_BASS_ADDRESS,0x13);		d();
	sendCommand(LG_BASS_ADDRESS,0x10);		d();
	
	sendCommand(LG_TREBLE_ADDRESS,0x11);	d();	//  3-    .
	sendCommand(LG_TREBLE_ADDRESS,0x10);	d();
	sendCommand(LG_TREBLE_ADDRESS,0x11);	d();
	sendCommand(LG_TREBLE_ADDRESS,0x27);	d();
	sendCommand(LG_TREBLE_ADDRESS,0x12);	d();
	sendCommand(LG_TREBLE_ADDRESS,0x13);	d();
	sendCommand(LG_TREBLE_ADDRESS,0x10);	d();

	sendCommand(RELE3_ADDRESS,0x21);		d();	//   3-.
	sendCommand(RELE3_ADDRESS,0x12);		d();
	sendCommand(RELE3_ADDRESS,0x13);		d();
	sendCommand(RELE3_ADDRESS,0x22);		d();
	sendCommand(RELE3_ADDRESS,0x12);		d();
	sendCommand(RELE3_ADDRESS,0x13);		d();
	sendCommand(RELE3_ADDRESS,0x24);		d();
	sendCommand(RELE3_ADDRESS,0x12);		d();
	sendCommand(RELE3_ADDRESS,0x13);		d();
	
	sendCommand(MOTOR_ADDRESS,0x21);		d();	//   .
	sendCommand(MOTOR_ADDRESS,0x12);		d();
	sendCommand(MOTOR_ADDRESS,0x13);		d();
	sendCommand(MOTOR_ADDRESS,0x11);		d();
	sendCommand(MOTOR_ADDRESS,0x22);		d();
	sendCommand(MOTOR_ADDRESS,0x12);		d();
	sendCommand(MOTOR_ADDRESS,0x13);		d();
	sendCommand(MOTOR_ADDRESS,0x11);		d();
	sendCommand(MOTOR_ADDRESS,0x24);		d();
	sendCommand(MOTOR_ADDRESS,0x12);		d();
	sendCommand(MOTOR_ADDRESS,0x13);		d();
	sendCommand(MOTOR_ADDRESS,0x11);		d();
	
	sendCommand(ONOFF_ADDRESS,0x11);		d();	//   
	sendCommand(ONOFF_ADDRESS,0x10);		d();
	
	goto xxx;*/
	
	GIMSK = 0b00100000; //      ().
	PCMSK = 1 << PB4;	//      !

#ifdef SONY
	TIMSK = 0b00000010;	//       0.
#else	
	OCR1A = 222;		//        RC-5.
	TCCR1 = 0b10000000;	//   1       OCR1A.
	TIMSK = 0b01000010;	//     OCR   1,    -   0.
#endif	
	
	loadTemplates();	//   .

	GIFR = 0b00100000; //      ().
	TIFR = 0b01000010;	//    .

	TCCR0A= 0x00;		//      .
	TCCR0B= 0x05;
	sei();

	if (templ[0] == 0xFFFF || (PINB & (1 << PB3)) == 0)	//     (  )     .
		{for(index = 0; index < sizeof(templ)/sizeof(templ[0]); index++)	//   - ,   .
			{indicateTemplNumber(index+1);		received = false;
			 do{while (!received);		newPackage = package;	received = false;
/*				for (int xxx = 0; xxx < 16; xxx++)			//      ( ).
					{indicateTemplNumber(0);	d();
					 if (newPackage & (1 << (15-xxx))) indicateTemplNumber(3); else indicateTemplNumber(1); d();
					}
				indicateTemplNumber(0);
				d();	d();*/
			   } while (newPackage == oldPackage);
			   
			 templ[index] = oldPackage = newPackage;
			}	
		 saveTemplates();		indicateTemplNumber(0x00);
		}

	cli();	mainTimer = 30 * TO_SECOND;	sei();
	for(received = false;;)		//     .
		if (received)
			{newPackage = package;	received = false;
			 for (index = 0; index < sizeof(templ)/sizeof(templ[0]); index++)
				if (newPackage == templ[index])	//  .
					{if (index != oldTerminal)		//    .
						{cli();	delayTimer = 0;	sei();
						 if (oldTerminal != 0xFF) automat(oldTerminal|T_UP);
						 oldTerminal = index;		automat(index);
						}
					 else //    .
						{cli();	delayTimer = TO_SECOND/2;	sei();
						 automat(index|T_REPEAT);	//      ...
						}
					 break;
					}
			}
		else if (delay)	//   .
			{delay = false;	automat(oldTerminal|T_UP);	oldTerminal = 0xFF;}
		else if (timeout)	//  .
			{timeout = false;	automat(T_TIMEOUT);}

	return 0;
}
